% Programa para procesado datos diagramas de radiación

clear all;
close all;

% Arrays para los datos de 0º a 345º

nivel_h = zeros(360/15,1601);
nivel_v = zeros(360/15,1601);
nivel_h_interp = zeros(360/15,1700-30+1);
nivel_v_interp = zeros(360/15,1700-30+1);

% Frecuencias y angulos

frecuencias_MHz = linspace(30, 1700, 1601);
frecuencias_MHz_interpoladas = 30:1:1700;
angulos = 0:15:360;


% Importa ganancia antena patron
datos_ganancia = importa_ganancia_patron(".\DS03_CBL6143A_23164_10D.TXT", [7, Inf]);

% interpola datos ganancia
ganancia_dBi = interp1(datos_ganancia(:,1),datos_ganancia(:,5), frecuencias_MHz); 

% Importar cada corte h y v

for i=1:(360/15)

    nombre = num2str((i-1)*15);
    
    ruta_h = strcat('.\h\',nombre,'.cti');
    ruta_v = strcat('.\v\',nombre,'.cti');
        
    nivel_h(i,:) = importar_corte_diagrama(ruta_h, [1613, 3213]);
    nivel_v(i,:) = importar_corte_diagrama(ruta_v, [1613, 3213]);
    
    % interpola datos para tener datos cada 1 MHz exacto

    nivel_h_interp(i,:) = interp1( frecuencias_MHz, nivel_h(i,:), frecuencias_MHz_interpoladas); 
    nivel_v_interp(i,:) = interp1( frecuencias_MHz, nivel_v(i,:), frecuencias_MHz_interpoladas); 
    
end

% Completar corte para que 0º sea igual a 360º
nivel_h_interp = [nivel_h_interp; nivel_h_interp(1,:)];
nivel_v_interp = [nivel_v_interp; nivel_v_interp(1,:)];    


% Calculo de los diagramas de radiación normalizados al valor del maximo
% para cada frecuencia (1MHz)

% Normalizar al valor del maximo de cada frecuencia ( no siempre coincide
% con azimut = 0, pero casi siempre)



for i=1:length(frecuencias_MHz_interpoladas)
    
    nivel_h_interp(:,i) = nivel_h_interp(:,i) - max( nivel_h_interp(:,i));
    nivel_v_interp(:,i) = nivel_v_interp(:,i) - max( nivel_v_interp(:,i));
end



% Representa el diagrama para cualquier frecuencia  
% Ejemplo frecuencia de 110MHz plano E y Plano H

% prompt = 'Introduce un valor de frecuencia en MHz para calcular el diagrama:';
% frecuencia_MHz_deseada = input(prompt);

while 1
frecuencia_MHz_deseada = input('Please, enter an integer frequency value in MHz to plot its diagram (30 to 1700):','s');
    if  round(str2num(frecuencia_MHz_deseada))<30 || round(str2num(frecuencia_MHz_deseada))>1700
        disp('Please, between 30 and 1700')
    else
        break;
    end
end


% Busca el valor más proximo al interpolado cada 1MHz
indice = knnsearch(frecuencias_MHz_interpoladas',round(str2num(frecuencia_MHz_deseada)));
valor_mostrado = frecuencias_MHz_interpoladas(indice);

figure(1)

phej = polarpattern(angulos,nivel_h_interp(:,indice));
hold on
polarpattern(angulos,nivel_v_interp(:,indice));

legend ('H. Pol. (E-Plane)', 'V. Pol. (H-Plane)','Location','Southeast');
%phej.LegendLabels={'Pol.Horizontal - Plano E','Pol. Vertical-Plano H'};
phej.LineWidth = 0.01;
phej.MagnitudeLim = [-40,0];
phej.AngleAtTop = 0; 
phej.TitleTop = strcat('Normalized radiation pattern (dB)');
phej.TitleBottom = strcat('Frequency: ', num2str(valor_mostrado),' MHz');
%phej.TitleBottom = 'Frecuencia 110 MHz';



% Representa los diagramas en incrementos definidos de frecuencia  
% Todas las frecuencias en pasos de X MHz para Plano E (Horizontal)
% prompt = 'Introduce un valor de incremento de frecuencia en MHz para calcular los diagramas (entre 1 y 1000):';
% incremento_frecuencia_diagrama_MHz = input(prompt);


while 1
incremento_frecuencia_diagrama_MHz = input('Now, to plot the diagram for a set of frequencies, please enter the interval in MHz between 1 and 1000 (e.g. 50):','s');
    if  round(str2num(incremento_frecuencia_diagrama_MHz))<1 || round(str2num(incremento_frecuencia_diagrama_MHz))>1000
        disp('Please, between 1 and 1000')
    else
        break;
    end
end

incremento_frecuencia_diagrama_MHz = round(str2num(incremento_frecuencia_diagrama_MHz));


figure(2)
% Desde el primer valor de frecuencia 30MHz en incrementos introducidos por
% el usuario hasta el ultimo valor
indices_frecuencias = 30:incremento_frecuencia_diagrama_MHz:1700;
% Se añade la ultima frecuencia medida
indices_frecuencias = [indices_frecuencias 1700]; % Porque empieza en 30MHz
indices_graficas = indices_frecuencias-29; % Porque empieza en 30MHz
numero_graficas = length(indices_graficas); 
leyenda = cell(1,numero_graficas);


for i=1:numero_graficas
       
    leyenda{1,i} = strcat( num2str(indices_frecuencias(i)), ' MHz' ); 
    % Representa
    ph = polarpattern(angulos,nivel_h_interp(:,indices_graficas(i)));
    hold on;

end

ph.LegendLabels = leyenda;
ph.LineWidth = 0.01;
ph.LegendVisible = 1;
ph.AngleAtTop = 0; 
ph.MagnitudeLim = [-40,0];
ph.TitleTop = strcat('Normalized radiation pattern (dB)');
ph.TitleBottom = strcat('Horizontal Polarization - E-Plane');


figure(3)
% Todas las frecuencias eb pasos de 50 MHz para PLano H (vertical)


for i=1:numero_graficas
       
    leyenda{1,i} = strcat( num2str(indices_frecuencias(i)), ' MHz' ); 
    % Representa
    pv = polarpattern(angulos,nivel_v_interp(:,indices_graficas(i)));
    hold on;

end

pv.LegendLabels = leyenda;
pv.LineWidth = 0.01;
pv.LegendVisible = 1;
pv.AngleAtTop = 0; 
pv.MagnitudeLim = [-40,0];
pv.TitleTop = strcat('Normalized radiation pattern (dB)');
pv.TitleBottom = strcat('Vertical Polarization - H-Plane');


% Calcula ganancia
% importa medida de referencia sólo se hace en polarización vertical
ruta_ref_v = strcat('.\v\referencia_0_v.cti');
ruta_ref_h = strcat('.\h\referencia_0_h.cti');

referencia_v = importar_corte_diagrama(ruta_ref_v, [1613, 3213]);   
referencia_h = importar_corte_diagrama(ruta_ref_h, [1613, 3213]);      

% calculo ganancia metodo de sustitución por referencia patrón conocida
  
Ganancia_abt_v = ganancia_dBi + nivel_v(1,:) - referencia_v';
Ganancia_abt_h = ganancia_dBi + nivel_h(1,:) - referencia_h';


% Representar ganancia vs frecuencia

figure(4)

semilogx(frecuencias_MHz,Ganancia_abt_v)
title('Gain (dBi) at 0º Azimuth (Vertical Polarization, H-Plane)')
xlabel('Frequency (MHz)')
ylabel('Gain (dBi)')

set(gca,'XTick',[30 50 100 1000 1700])
set(gca,'XTickLabel',{'30','50','100','1000','1700'})
hAx=gca;  % avoid repetitive function calls
set(hAx,'xminorgrid','on','yminorgrid','on')

hold on;
box on;
grid on;


figure(5)

semilogx(frecuencias_MHz,Ganancia_abt_h)
title('Gain (dBi) at 0º Azimuth (Horizontal Polarization, E-Plane)')
xlabel('Frequency (MHz)')
ylabel('Gain (dBi)')

set(gca,'XTick',[30 50 100 1000 1700])
set(gca,'XTickLabel',{'30','50','100','1000','1700'})
hAx=gca;  % avoid repetitive function calls
set(hAx,'xminorgrid','on','yminorgrid','on')

hold on;
box on;
grid on;





